@@ -14,15 +14,7 @@ const { existsSync } = require("node:fs");
 const path = require("node:path");
 const { version } = require("../../package.json");
 const { Linter } = require("../linter");
-const { getRuleFromConfig } = require("../config/flat-config-helpers");
 const { defaultConfig } = require("../config/default-config");
-const {
-	Legacy: {
-		ConfigOps: { getRuleSeverity },
-		ModuleResolver,
-		naming,
-	},
-} = require("@eslint/eslintrc");
 
 const {
 	findFiles,
@@ -41,6 +33,14 @@ const { pathToFileURL } = require("node:url");
 const LintResultCache = require("../cli-engine/lint-result-cache");
 const { Retrier } = require("@humanwhocodes/retry");
 const { ConfigLoader, LegacyConfigLoader } = require("../config/config-loader");
+const { WarningService } = require("../services/warning-service");
+const { Config } = require("../config/config.js");
+const {
+	getShorthandName,
+	getNamespaceFromTerm,
+	normalizePackageName,
+} = require("../shared/naming.js");
+const { resolve } = require("../shared/relative-module-resolver.js");
 
 /*
  * This is necessary to allow overwriting writeFile for testing purposes.
@@ -57,16 +57,21 @@ const { ConfigLoader, LegacyConfigLoader } = require("../config/config-loader");
  * @import { CLIEngineLintReport } from "./legacy-eslint.js";
  * @import { FlatConfigArray } from "../config/flat-config-array.js";
  * @import { RuleDefinition } from "@eslint/core";
- * @import { ConfigData, DeprecatedRuleInfo, LintMessage, LintResult, Plugin, ResultsMeta } from "../shared/types.js";
  */
 
 /** @typedef {ReturnType<ConfigArray.extractConfig>} ExtractedConfig */
+/** @typedef {import("../types").Linter.Config} Config */
+/** @typedef {import("../types").ESLint.DeprecatedRuleUse} DeprecatedRuleInfo */
+/** @typedef {import("../types").Linter.LintMessage} LintMessage */
+/** @typedef {import("../types").ESLint.LintResult} LintResult */
+/** @typedef {import("../types").ESLint.Plugin} Plugin */
+/** @typedef {import("../types").ESLint.ResultsMeta} ResultsMeta */
 
 /**
  * The options with which to configure the ESLint instance.
  * @typedef {Object} ESLintOptions
  * @property {boolean} [allowInlineConfig] Enable or disable inline configuration comments.
- * @property {ConfigData|Array<ConfigData>} [baseConfig] Base config, extended by all configs used with this instance
+ * @property {Config|Array<Config>} [baseConfig] Base config, extended by all configs used with this instance
  * @property {boolean} [cache] Enable result caching.
  * @property {string} [cacheLocation] The cache file to use instead of .eslintcache.
  * @property {"metadata" | "content"} [cacheStrategy] The strategy used to detect changed files.
@@ -78,7 +83,7 @@ const { ConfigLoader, LegacyConfigLoader } = require("../config/config-loader");
  * @property {boolean} [globInputPaths] Set to false to skip glob resolution of input file paths to lint (default: true). If false, each input file paths is assumed to be a non-glob path to an existing file.
  * @property {boolean} [ignore] False disables all ignore patterns except for the default ones.
  * @property {string[]} [ignorePatterns] Ignore file patterns to use in addition to config ignores. These patterns are relative to `cwd`.
- * @property {ConfigData|Array<ConfigData>} [overrideConfig] Override config, overrides all configs used with this instance
+ * @property {Config|Array<Config>} [overrideConfig] Override config, overrides all configs used with this instance
  * @property {boolean|string} [overrideConfigFile] Searches for default config file when falsy;
  *      doesn't do any config file lookup when `true`; considered to be a config filename
  *      when a string.
@@ -155,10 +160,10 @@ function getOrFindUsedDeprecatedRules(eslint, maybeFilePath) {
 
 		if (config.rules) {
 			for (const [ruleId, ruleConf] of Object.entries(config.rules)) {
-				if (getRuleSeverity(ruleConf) === 0) {
+				if (Config.getRuleNumericSeverity(ruleConf) === 0) {
 					continue;
 				}
-				const rule = getRuleFromConfig(ruleId, config);
+				const rule = config.getRuleDefinition(ruleId);
 				const meta = rule && rule.meta;
 
 				if (meta && meta.deprecated) {
@@ -169,7 +174,7 @@ function getOrFindUsedDeprecatedRules(eslint, maybeFilePath) {
 						replacedBy: usesNewFormat
 							? (meta.deprecated.replacedBy?.map(
 									replacement =>
-										`${replacement.plugin?.name !== void 0 ? `${naming.getShorthandName(replacement.plugin.name, "eslint-plugin")}/` : ""}${replacement.rule?.name ?? ""}`,
+										`${replacement.plugin?.name !== void 0 ? `${getShorthandName(replacement.plugin.name, "eslint-plugin")}/` : ""}${replacement.rule?.name ?? ""}`,
 								) ?? [])
 							: meta.replacedBy || [],
 						info: usesNewFormat ? meta.deprecated : void 0,
@@ -352,7 +357,7 @@ function shouldMessageBeFixed(message, config, fixTypes) {
 		return fixTypes.has("directive");
 	}
 
-	const rule = message.ruleId && getRuleFromConfig(message.ruleId, config);
+	const rule = message.ruleId && config.getRuleDefinition(message.ruleId);
 
 	return Boolean(rule && rule.meta && fixTypes.has(rule.meta.type));
 }
@@ -386,6 +391,20 @@ function getFixerForFixTypes(fix, fixTypesSet, config) {
 		originalFix(message);
 }
 
+/**
+ * Retrieves flags from the environment variable ESLINT_FLAGS.
+ * @param {string[]} flags The flags defined via the API.
+ * @returns {string[]} The merged flags to use.
+ */
+function mergeEnvironmentFlags(flags) {
+	if (!process.env.ESLINT_FLAGS) {
+		return flags;
+	}
+
+	const envFlags = process.env.ESLINT_FLAGS.trim().split(/\s*,\s*/gu);
+	return Array.from(new Set([...envFlags, ...flags]));
+}
+
 //-----------------------------------------------------------------------------
 // Main API
 //-----------------------------------------------------------------------------
@@ -413,10 +432,12 @@ class ESLint {
 	constructor(options = {}) {
 		const defaultConfigs = [];
 		const processedOptions = processOptions(options);
+		const warningService = new WarningService();
 		const linter = new Linter({
 			cwd: processedOptions.cwd,
 			configType: "flat",
-			flags: processedOptions.flags,
+			flags: mergeEnvironmentFlags(processedOptions.flags),
+			warningService,
 		});
 
 		const cacheFilePath = getCacheFile(
@@ -439,6 +460,7 @@ class ESLint {
 			hasUnstableNativeNodeJsTSConfigFlag: linter.hasFlag(
 				"unstable_native_nodejs_ts_config",
 			),
+			warningService,
 		};
 
 		this.#configLoader = linter.hasFlag("unstable_config_lookup_from_file")
@@ -467,8 +489,7 @@ class ESLint {
 			for (const [pluginName, plugin] of Object.entries(
 				options.plugins,
 			)) {
-				plugins[naming.getShorthandName(pluginName, "eslint-plugin")] =
-					plugin;
+				plugins[getShorthandName(pluginName, "eslint-plugin")] = plugin;
 			}
 
 			defaultConfigs.push({
@@ -478,10 +499,7 @@ class ESLint {
 
 		// Check for the .eslintignore file, and warn if it's present.
 		if (existsSync(path.resolve(processedOptions.cwd, ".eslintignore"))) {
-			process.emitWarning(
-				'The ".eslintignore" file is no longer supported. Switch to using the "ignores" property in "eslint.config.js": https://eslint.org/docs/latest/use/configure/migration-guide#ignoring-files',
-				"ESLintIgnoreWarning",
-			);
+			warningService.emitESLintIgnoreWarning();
 		}
 	}
 
@@ -610,7 +628,7 @@ class ESLint {
 				if (!config) {
 					throw createExtraneousResultsError();
 				}
-				const rule = getRuleFromConfig(ruleId, config);
+				const rule = config.getRuleDefinition(ruleId);
 
 				// ignore unknown rules
 				if (rule) {
@@ -703,15 +721,11 @@ class ESLint {
 			debug(`Deleting cache file at ${cacheFilePath}`);
 
 			try {
-				await fs.unlink(cacheFilePath);
+				if (existsSync(cacheFilePath)) {
+					await fs.unlink(cacheFilePath);
+				}
 			} catch (error) {
-				const errorCode = error && error.code;
-
-				// Ignore errors when no such file exists or file system is read only (and cache file does not exist)
-				if (
-					errorCode !== "ENOENT" &&
-					!(errorCode === "EROFS" && !existsSync(cacheFilePath))
-				) {
+				if (existsSync(cacheFilePath)) {
 					throw error;
 				}
 			}
@@ -983,7 +997,7 @@ class ESLint {
 
 		// replace \ with / for Windows compatibility
 		const normalizedFormatName = name.replace(/\\/gu, "/");
-		const namespace = naming.getNamespaceFromTerm(normalizedFormatName);
+		const namespace = getNamespaceFromTerm(normalizedFormatName);
 
 		// grab our options
 		const { cwd } = privateMembers.get(this).options;
@@ -995,16 +1009,13 @@ class ESLint {
 			formatterPath = path.resolve(cwd, normalizedFormatName);
 		} else {
 			try {
-				const npmFormat = naming.normalizePackageName(
+				const npmFormat = normalizePackageName(
 					normalizedFormatName,
 					"eslint-formatter",
 				);
 
 				// TODO: This is pretty dirty...would be nice to clean up at some point.
-				formatterPath = ModuleResolver.resolve(
-					npmFormat,
-					getPlaceholderPath(cwd),
-				);
+				formatterPath = resolve(npmFormat, getPlaceholderPath(cwd));
 			} catch {
 				formatterPath = path.resolve(
 					__dirname,
@@ -1071,7 +1082,7 @@ class ESLint {
 	 * This is the same logic used by the ESLint CLI executable to determine
 	 * configuration for each file it processes.
 	 * @param {string} filePath The path of the file to retrieve a config object for.
-	 * @returns {Promise<ConfigData|undefined>} A configuration object for the file
+	 * @returns {Promise<Config|undefined>} A configuration object for the file
 	 *      or `undefined` if there is no configuration data for the object.
 	 */
 	async calculateConfigForFile(filePath) {
